#!/usr/bin/perl

use strict;

#  ---------------------------------------------------------------------
#   simple user interface for IRDB component
#    v1.0
#    February 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu
#     Digital Library Research Laboratory   |       www.dlib.vt.edu
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu
#  -------------------------------------------------------+-------------


use FindBin;
use lib "$FindBin::Bin/lib";


use Pure::EZCGI;
use Pure::EZXML;
use XOAI::XOAISP;


# set the baseURL of the IRDB search engine component
my $baseURL = 'http://oai.dlib.vt.edu/~hussein/cgi-bin/compute/ODL-IRDB/IRDB/compute/search.pl';


# generate an HTML prolog and header
sub header
{
   "Content-type: text/html\n\n".
   "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n".
   "<html>\n\n".
   "<head>\n".
   "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">".
   "<title>COMPUTE Search</title>\n".
   "</head>\n\n".
   "<body>\n".
   "<h1><center><font color=\"#88000\">".
   "COMPUTE Search Engine\n".
   "</font></center></h1><hr>\n";
}


# generate an HTML footer
sub footer
{
   "<hr><i><center>Send all queries to hussein\@vt.edu</center></i>\n".
   "</body>\n".
   "</html>\n";
}


# generate a search form, given the last query issued
sub SearchForm
{
   my ($query) = @_;

   "This is an experimental search engine that searches through a union ".
   "of the NCSTRL-Historical, CogPrints, arXiv-cs and CSTC collections\n".
   "<p>".

   "<center><form method=\"get\" action=\"#\">".
   "Search Terms : ".
   "<input type=text name=query value=\"$query\" size=40> ".
   "<input type=submit value=\"Go\">".
   "</form></center><hr>";
}


# generate a button to get more results, either next or previous
sub MoreButton
{
   my ($button, $query, $start) = @_;
   
   "<form method=\"get\" action=\"#\">".
   "<input type=hidden name=query value=\"$query\">".
   "<input type=hidden name=start value=\"$start\">".
   "<input type=submit value=\"$button\">".
   "</form>";
}


# format the output from IRDB for display, using only the XML::DOM parser
sub Format
{
   my ($start, $data, $query, $pagecount) = @_;

   # parse XML
   my $parser = new Pure::EZXML;
   my $document = $parser->parse ($data)->getDocumentElement;
   
   my $buffer = "<ol start=\"$start\">";
   
   # mappings from ids to human-readable names and URLs
   my %name = ( 'cogprints' => 'Cogprints', 'CSTC' => 'Computer Science Teaching Center',
                'arXiv' => 'arXiv', 'ncstrlh' => 'NCSTRL Historical Collection' );
   my %URL = ( 'cogprints' => 'http://www.cogprints.org', 'CSTC' => 'http://www.cstc.org',
               'arXiv' => 'http://arXiv.org', 'ncstrlh' => 'http://www.ncstrl.org' );

   # get the total number of hits
   my $hits = $document->getElementsByTagName ('responseContainer', 0)
     ->item(0)->getElementsByTagName ('hits', 0)->item(0)->getChildNodes->toString;
   
   # go through each record in the list
   foreach my $record ($document->getElementsByTagName ('record', 0))
   {
      my @metadata = $record->getElementsByTagName ('metadata', 0);
      if ($#metadata > -1)
      {
         my @dc = $metadata[0]->getElementsByTagName ('dc', 0);
         if ($#dc == -1)
         {
            @dc = $metadata[0]->getElementsByTagName ('oai_dc', 0);
         }
         
         # if a Dublin Core record does exist, get and output fields
         if ($#dc > -1)
         {
            $buffer .= "<li><font size=\"+1\" face=\"arial\">";
            my @title = $dc[0]->getElementsByTagName ('title', 0);
            my @creator = $dc[0]->getElementsByTagName ('creator', 0);
            my @date = $dc[0]->getElementsByTagName ('date', 0);
            my @identifier = $dc[0]->getElementsByTagName ('identifier', 0);
            foreach my $t (@title)
            {
               if ($#identifier > -1)
               {
                  $buffer .= "<a href=\"".$identifier[0]->getChildNodes->toString."\">";
               }
               $buffer .= $t->getChildNodes->toString;
               if ($#identifier > -1)
               {
                  $buffer .= '</a>';
               }
               $buffer .= '<br>';
            }
            $buffer .= "</font><i><font size=\"-1\">";
            my $first = 1;
            if ($#creator > -1)
            {
               $buffer .= 'By : ';
               foreach my $c (@creator)
               {
                  if ($first == 1)
                  { $first = 0; }
                  else
                  { $buffer .= ', '; }
                  my $author = $c->getChildNodes->toString;
                  if ($author =~ /^([^,]+),\s*(.+)$/)
                  {
                     $author = "$2 $1";
                  }
                  $buffer .= $author;
               }
            }
            foreach my $d (@date)
            {
               if ($first == 1)
               { $first = 0; }
               else
               { $buffer .= ', '; }
               $buffer .= ' '.$d->getChildNodes->toString;
            }
            
            # map source from identifier
            my $id = $record->getElementsByTagName ('header')->item(0)
              ->getElementsByTagName ('identifier')->item(0)->getChildNodes->toString;
            if ($id =~ /oai:(cogprints|ncstrlh|CSTC|arXiv)(.*)/)
            {
               $buffer .= " (Source: <a href=\"$URL{$1}\" title=\"OAI identifier: $id\">$name{$1}</a>)";
            }
            $buffer .= '</font></i></li><p>';
         }
      }
   }
   $buffer .= "</ol><center><table width=\"50%\"><tr>";
   
   # if necessary, output previous and next buttons
   if ($start > 1)
   {
      $buffer .= "<td align=\"left\">".
                 MoreButton ('Previous Page << ', $query, $start-$pagecount).
                 '</td>';
   }
   $buffer .= ' ';
   if ($start+$pagecount-1 < $hits)
   {
      $buffer .= "<td align=\"right\">".
                 MoreButton ('>> Next Page', $query, $start+$pagecount).
                 '</td>';
   }

   $buffer .= '</tr></table></center>';
}


# main program body
sub main
{
   print header;

   my $cgi = new Pure::EZCGI;

   my $start = $cgi->param ('start');
   my $query = $cgi->param ('query');
   my $pagecount = 20;
   
   print SearchForm ($query);

   if ($query ne '')
   {
      # create service provider object
      my $xoai = new XOAI::XOAISP ($baseURL);
   
      my $querystring = $query;
   
      # set number of results being requested
      if ($start eq '')
      {
         $start = 1;
      }
      my $stop = $start+$pagecount-1;

      # issue query, format output and print
      $query =~ s/ /%20/go;
      my $outputdata = $xoai->doXOAI ('ListRecords', 
         { set=> "odlsearch1/$query/$start/$stop",
           metadataPrefix => 'oai_dc',
         })->content;
      print Format ($start, $outputdata, $querystring, $pagecount);
   }

   print footer;
}

main;
